/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include  "../../project_config.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************
extern ATCAIfaceCfg sha104_0_init_data;
extern ATCAIfaceCfg sha104_1_init_data;
extern ATCAIfaceCfg sha105_0_init_data;
extern uint8_t fixed_data[];
extern uint8_t other_data[];

#define MAC_MODE 0x40
#define SYMM_SLOT 3

uint8_t host_challenge[RANDOM_NUM_SIZE];
uint8_t accessory_mac[MAC_SIZE];
uint8_t accessory_sn[ATCA_SERIAL_NUM_SIZE];
char displaystr[200];
size_t displaylen;

ATCAIfaceCfg *sha104_cfg = (SELECTED_ECC204_SWI) ? &sha104_1_init_data : &sha104_0_init_data;
// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;
// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/
ATCA_STATUS generate_challenge_on_host(void)
{
    ATCA_STATUS status;
    uint8_t num_in[NONCE_NUMIN_SIZE];

    if (ATCA_SUCCESS != (status = atcab_init(&sha105_0_init_data)))
    {
        printf("atcab_init(sha105_0) has failed with status {%02X}.\r\n", status);
        return status;
    }

    if (ATCA_SUCCESS != (status = atcab_nonce_rand(num_in, host_challenge)))
    {
        printf("atcab_nonce_rand has failed with status {%02X}.\r\n", status);
        return status;
    }
    displaylen = sizeof(displaystr);
    atcab_bin2hex(host_challenge, sizeof(host_challenge), displaystr, &displaylen);
    printf("\nHost random:\r\n%s\r\n", displaystr);

    return status;
}

ATCA_STATUS get_mac_from_accessory(void)
{
    ATCA_STATUS status;
    uint32_t counter_pre_mac, counter_post_mac;

    if (ATCA_SUCCESS != (status = atcab_init(sha104_cfg)))
    {
        printf("atcab_init(sha104_0) has failed with status {%02X}.\r\n", status);
        return status;
    }

    // Reading the counter values to show it increments on MAC operation.
    // The counter increments only when it is attached to HMAC key
    if (ATCA_SUCCESS != (status = atcab_counter_read(0, &counter_pre_mac)))
    {
        printf("atcab_counter_read has failed with status {%02X}.\r\n", status);
        return status;
    }

    if (ATCA_SUCCESS != (status = atcab_mac(MAC_MODE_INCLUDE_SN, SYMM_SLOT, host_challenge, accessory_mac)))
    {
        printf("atcab_mac has failed with status {%02X}.\r\n", status);
        return status;
    }
    displaylen = sizeof(displaystr);
    atcab_bin2hex(accessory_mac, sizeof(accessory_mac), displaystr, &displaylen);
    printf("\nAccessory MAC:\r\n%s\r\n", displaystr);

    if (ATCA_SUCCESS != (status = atcab_counter_read(0, &counter_post_mac)))
    {
        printf("atcab_counter_read has failed with status {%02X}.\r\n", status);
        return status;
    }
    printf("\nAccessory Counter(pre_mac:post_mac):\r\n%ld:%ld\r\n", counter_pre_mac, counter_post_mac);

    // Reading accessory serial number to use with host secure element for key diversification
    if (ATCA_SUCCESS != (status = atcab_read_serial_number(accessory_sn)))
    {
        printf("atcab_read_serial_number has failed with status {%02X}.\r\n", status);
        return status;
    }
    displaylen = sizeof(displaystr);
    atcab_bin2hex(accessory_sn, sizeof(accessory_sn), displaystr, &displaylen);
    printf("\nAccessory SNo:\r\n%s\r\n", displaystr);

    return status;
}

ATCA_STATUS verify_mac_with_host_checkmac(void)
{
    ATCA_STATUS status;
    uint8_t gendivkey_fixed_data[32]; // Serial number (9) + 0 (23)
    uint8_t checkmac_other_data[13];
    uint8_t checkmac_mode;

    if (ATCA_SUCCESS != (status = atcab_init(&sha105_0_init_data)))
    {
        printf("atcab_init(sha104_0) has failed with status {%02X}.\r\n", status);
        return status;
    }

    memcpy(gendivkey_fixed_data, accessory_sn, sizeof(accessory_sn));
    memcpy(&gendivkey_fixed_data[sizeof(accessory_sn)], fixed_data, sizeof(fixed_data));
    displaylen = sizeof(displaystr);
    atcab_bin2hex(gendivkey_fixed_data, sizeof(gendivkey_fixed_data), displaystr, &displaylen);
    printf("\nFixed data for GenDivKey:\r\n%s\r\n", displaystr);
    displaylen = sizeof(displaystr);
    atcab_bin2hex(other_data, sizeof(other_data), displaystr, &displaylen);
    printf("Other data for GenDivKey:\r\n%s\r\n", displaystr);

    if(ATCA_SUCCESS != (status = atcab_nonce_base(NONCE_MODE_PASSTHROUGH, 0, gendivkey_fixed_data, NULL)))
    {
        printf("atcab_nonce_base has failed with status {%02X}.\r\n", status);
        return status;
    }

    if(ATCA_SUCCESS != (status = atcab_gendivkey(other_data)))
    {
        printf("atcab_gendivkey has failed with status {%02X}.\r\n", status);
        return status;
    }

    checkmac_other_data[0] = ATCA_MAC;
    checkmac_other_data[1] = MAC_MODE_INCLUDE_SN;
    checkmac_other_data[2] = SYMM_SLOT;
    memset(&checkmac_other_data[3], 0, 4);
    memcpy(&checkmac_other_data[7], &accessory_sn[4], 4);
    memcpy(&checkmac_other_data[11], &accessory_sn[2], 2);

    printf("\nPerforming authentication with Checkmac...\r\n");
    checkmac_mode = CHECKMAC_MODE_BLOCK1_TEMPKEY|CHECKMAC_MODE_SOURCE_FLAG_MATCH;
    if(ATCA_SUCCESS != (status = atcab_checkmac(checkmac_mode, SYMM_SLOT, host_challenge,
                                        accessory_mac, checkmac_other_data)))
    {
        printf("atcab_checkmac has failed with status {%02X}.\r\n", status);
        return status;
    }

    printf("\nAccessory / Disposable authentication is successful using mac-checkmac.\r\n");
    printf("\n*********");

    return status;
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
          bool appInitialized = true;
          if (appInitialized)
          {
            appData.state = APP_STATE_SERVICE_TASKS;
          }
          break;
        }

        case APP_STATE_SERVICE_TASKS:
          printf("\r\n\r\n********* Running Symm Auth usecase firmware....\r\n");
          appData.state = APP_STATE_GENERATE_CHALLENGE_ON_HOST;
          break;

        case APP_STATE_GENERATE_CHALLENGE_ON_HOST:
          appData.state = (generate_challenge_on_host() == ATCA_SUCCESS) ? APP_STATE_GET_MAC_FROM_ACCESSORY : APP_STATE_FINISH;
          break;

        case APP_STATE_GET_MAC_FROM_ACCESSORY:
          appData.state = (get_mac_from_accessory() == ATCA_SUCCESS) ? APP_STATE_VERIFY_MAC_WITH_HOST_CHECKMAC : APP_STATE_FINISH;
          break;

        case APP_STATE_VERIFY_MAC_WITH_HOST_CHECKMAC:
          appData.state = (verify_mac_with_host_checkmac() == ATCA_SUCCESS) ? APP_STATE_FINISH : APP_STATE_FINISH;
          break;

        /* TODO: implement your application state machine.*/
        case APP_STATE_FINISH:
          break;

        /* The default state should never be executed. */
        default:
        {
          /* TODO: Handle error in application's state machine. */
          break;
        }
    }
}


/*******************************************************************************
 End of File
 */
